/**
 * EasyBeans
 * Copyright (C) 2006-2007 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: StatefulSessionFactory.java 4463 2009-01-21 13:52:01Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.container.session.stateful;

import static org.ow2.easybeans.api.OperationState.BUSINESS_METHOD;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import javax.ejb.ApplicationException;
import javax.ejb.EJBException;
import javax.ejb.NoSuchEJBException;
import javax.ejb.Timer;

import org.ow2.easybeans.api.EZBContainer;
import org.ow2.easybeans.api.FactoryException;
import org.ow2.easybeans.api.IStatefulSessionFactory;
import org.ow2.easybeans.api.OperationState;
import org.ow2.easybeans.api.bean.EasyBeansSFSB;
import org.ow2.easybeans.container.session.SessionFactory;
import org.ow2.easybeans.rpc.JEJBResponse;
import org.ow2.easybeans.rpc.api.EJBLocalRequest;
import org.ow2.easybeans.rpc.api.EJBResponse;
import org.ow2.easybeans.rpc.api.RPCException;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.pool.api.PoolException;
import org.ow2.util.pool.impl.JPool;
import org.ow2.util.pool.impl.PoolEntryStatistics;
import org.ow2.util.pool.impl.PoolFactory;

/**
 * This class manages the stateless session bean and its creation/lifecycle.
 * @author Florent Benoit
 */
public class StatefulSessionFactory extends SessionFactory<EasyBeansSFSB> implements PoolFactory<EasyBeansSFSB, Long>,
        IStatefulSessionFactory {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(StatefulSessionFactory.class);

    /**
     * Id generator.
     */
    private long idCount = 0L;

    /**
     * Builds a new factory with a given name and its container.
     * @param className name of this factory (name of class that is managed)
     * @param container the root component of this factory.
     * @throws FactoryException if class can't be loaded.
     */
    public StatefulSessionFactory(final String className, final EZBContainer container) throws FactoryException {
        super(className, container);
        JPool<EasyBeansSFSB, Long> pool = new JPool<EasyBeansSFSB, Long>(this);
        // stateful = only one client at a given time.
        pool.setAllowSharedInstance(false);
        setPool(pool);
    }

    /**
     * Checks if the given object with the given clue is matching.
     * @param bean given object against which the check should be done.
     * @param clue the object used as clue to check the matching.
     * @return true if it is matching, else false.
     */
    public boolean isMatching(final EasyBeansSFSB bean, final Long clue) {
        logger.debug("Called with bean = {0} and beanId = {1}", bean, clue);
        Long beanId = bean.getEasyBeansStatefulID();
        boolean val = (beanId.equals(clue));
        logger.debug("Found id = {0} and will return = {1}", beanId, Boolean.valueOf(val));
        return val;
    }

    /**
     * Validate an instance by giving some statistics.
     * @param object the instance to validate
     * @param stats some statistics to help in the validating process.
     * @return true if the element is valid, else false.
     */
    public boolean validate(final EasyBeansSFSB object, final PoolEntryStatistics stats) {
        return true;
    }

    /**
     * Gets a new ID or a null value.
     * @param beanId given id.
     * @return new id
     */
    @Override
    protected synchronized Long getId(final Long beanId) {
        Long newId = beanId;
        // no Id, compute a new one
        if (newId == null) {
            this.idCount++;
            newId = Long.valueOf(this.idCount);
        }
        return newId;
    }

    /**
     * Creates an instance with the given hint.
     * @param clue a clue given by the Pool. Could be null.
     * @throws PoolException if instance cannot be created.
     * @return the created instance.
     */
    @Override
    public synchronized EasyBeansSFSB create(final Long clue) throws PoolException {
        // else, create it
        EasyBeansSFSB bean = super.create(clue);

        return bean;
    }

    /**
     * Gets a bean for the given id.
     * @param beanId id of the expected bean.
     * @return a Stateless bean.
     * @throws IllegalArgumentException if bean is not found.
     */
    @Override
    protected synchronized EasyBeansSFSB getBean(final Long beanId) throws IllegalArgumentException {
        EasyBeansSFSB bean = null;
        try {
            bean = getPool().get(beanId);
        } catch (PoolException e) {
            throw new IllegalArgumentException("Cannot get element in the pool", e);
        }
        logger.debug("Set for bean {0} the Id = {1}", bean, beanId);
        bean.setEasyBeansStatefulID(beanId);
        return bean;
    }

    /**
     * Callback called when object is gonna be removed.
     * @param instance that is being removed from the pool.
     */
    @Override
    public void remove(final EasyBeansSFSB instance) {
        super.remove(instance);
        instance.setEasyBeansRemoved(true);
    }

    /**
     * Do a local call on a method of this factory.
     * @param localCallRequest the given request
     * @return response with the value of the call and the bean ID (if any)
     */
    @Override
    public EJBResponse localCall(final EJBLocalRequest localCallRequest) {
        Long id = getId(localCallRequest.getBeanId());

        // build EJB Response and set the id
        EJBResponse ejbResponse = new JEJBResponse();
        ejbResponse.setBeanId(id);

        EasyBeansSFSB bean = null;
        try {
            bean = getBean(id);
        } catch (IllegalArgumentException e) {
            ejbResponse.setRPCException(new RPCException("Cannot get element in the pool", e));
            return ejbResponse;
        } catch (NoSuchEJBException e) {
            ejbResponse.setRPCException(new RPCException("Bean has been removed", e));
            return ejbResponse;
        }

        Method m = getHashes().get(localCallRequest.getMethodHash());

        if (m == null) {
            ejbResponse.setRPCException(new RPCException("Cannot find method called on the bean '" + getClassName() + "'."));
            return ejbResponse;
        }

        Object value = null;

        // set ClassLoader
        ClassLoader oldClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(getContainer().getClassLoader());
        // Set InvokedBusiness Interface
        String oldInvokedBusinessInterface = getInvokedBusinessInterfaceNameThreadLocal().get();
        getInvokedBusinessInterfaceNameThreadLocal().set(localCallRequest.getInvokedBusinessInterfaceName());

        // Operation state
        OperationState oldState = getOperationState();
        getOperationStateThreadLocal().set(BUSINESS_METHOD);

        synchronized (bean) {
            try {
                value = m.invoke(bean, localCallRequest.getMethodArgs());
            } catch (IllegalArgumentException e) {
                ejbResponse.setRPCException(new RPCException(e));
            } catch (IllegalAccessException e) {
                ejbResponse.setRPCException(new RPCException(e));
            } catch (InvocationTargetException e) {
                Throwable cause = e.getCause();
                RPCException rpcException = new RPCException(cause);
                // ApplicationException ?
                ApplicationException applicationException = getBeanInfo().getApplicationExceptions().get(
                        cause.getClass().getName());
                if (applicationException != null) {
                    rpcException.setApplicationException();
                }
                ejbResponse.setRPCException(rpcException);
            } finally {
                Thread.currentThread().setContextClassLoader(oldClassLoader);
                getInvokedBusinessInterfaceNameThreadLocal().set(oldInvokedBusinessInterface);
                getOperationStateThreadLocal().set(oldState);

                // push back into the pool
                try {
                    getPool().release(bean);
                } catch (PoolException e) {
                    ejbResponse.setRPCException(new RPCException("cannot release bean", e));
                }

                // If the bean has been removed (stateful), flag it as a removed
                // bean, so the client won't call again any methods.
                if (bean.getEasyBeansRemoved()) {
                    ejbResponse.setRemoved(true);
                }

            }
        }
        ejbResponse.setValue(value);
        return ejbResponse;

    }

    /**
     * Notified when the timer service send a Timer object.
     * It has to call the Timed method.
     * @param timer the given timer object that will be given to the timer method.
     */
    public void notifyTimeout(final Timer timer) {
        throw new EJBException("Stateful bean cannot receive Timer objects");
    }
}
